/*
 * Copyright (c) 1996 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <DemVtk/demprobefilter.h>
#include <DemVtk/demtriangle.h>

DemProbeFilter::DemProbeFilter()
{
  this->Source = NULL;
}

void DemProbeFilter::Execute()
{
  int ptId;
  float *x, tol2;
  vtkCell *cell;
  vtkPointData *pd, *outPD;
  int numPts, subId;
  vtkDataSet *source=this->Source, *input=this->Input;
  vtkDataSet *output=this->Output;
  float pcoords[3], *weights=new float[source->GetMaxCellSize()];

  vtkDebugMacro(<<"Probing data");

  pd = source->GetPointData();
  numPts = input->GetNumberOfPoints();
//
// Allocate storage for output PointData
//
  outPD = output->GetPointData();
  outPD->InterpolateAllocate(pd);
  vtkFloatScalars* outSC = vtkFloatScalars::New();
  outSC->Allocate(outPD->GetScalars()->GetNumberOfScalars());
  outPD->SetScalars(outSC);
//
// Use tolerance as a function of size of source data
//
  tol2 = source->GetLength();
  tol2 = tol2*tol2 / 1000.0;
//
// Loop over all input points, interpolating source data
//
  for (ptId=0; ptId < numPts; ptId++)
    {
    // Get the xyz coordinate of the point in the input dataset
    x = input->GetPoint(ptId);

    // Find the cell that contains xyz and get it
    

    cell = source->FindAndGetCell(x,NULL,0,tol2,subId,pcoords,weights);
    if (cell)
      {
      // Interpolate the point data
	//      outPD->InterpolatePoint(pd,ptId,&(cell->PointIds),weights);
	DemTriangle* dt = (DemTriangle*)cell;
	outPD->GetScalars()->InsertScalar(ptId, dt->EvaluateZ(x));
      }
    else
      {
      outPD->NullPoint(ptId);
      //printf("oops, didn't find cell!\n");
      }
    }
  delete [] weights;
}

// Description:
// Overload update method because execution can branch two ways (Input 
// and Source). Also input and output are abstract.
void DemProbeFilter::Update()
{
  // make sure output has been created
  if ( !this->Output )
    {
    vtkErrorMacro(<< "No output has been created...need to set input");
    return;
    }

  // make sure input is available
  if ( this->Source == NULL || this->Input == NULL )
    {
    vtkErrorMacro(<< "No input...can't execute!");
    return;
    }

  // prevent chasing our tail
  if (this->Updating) return;

  this->Updating = 1;
  this->Source->Update();
  this->Input->Update();
  this->Updating = 0;

  if (this->Source->GetMTime() > this->ExecuteTime || 
  this->Input->GetMTime() > this->ExecuteTime || 
  this->GetMTime() > this->ExecuteTime )
    {
    if ( this->Input->GetDataReleased() ) this->Input->ForceUpdate();
    if ( this->Source->GetDataReleased() ) this->Source->ForceUpdate();

    if ( this->StartMethod ) (*this->StartMethod)(this->StartMethodArg);
    this->Output->CopyStructure(this->Input);
    this->Execute();
    this->ExecuteTime.Modified();
    this->SetDataReleased(0);
    if ( this->EndMethod ) (*this->EndMethod)(this->EndMethodArg);
    }

  if ( this->Source->ShouldIReleaseData() ) this->Source->ReleaseData();
  if ( this->Input->ShouldIReleaseData() ) this->Input->ReleaseData();
}

void DemProbeFilter::PrintSelf(ostream& os, vtkIndent indent)
{
  vtkDataSetToDataSetFilter::PrintSelf(os,indent);

  os << indent << "Source: " << this->Source << "\n";
}
